@@ -141,21 +141,6 @@ add_env (GPtrArray  *array,
               NULL);
 }
 
-static char *
-get_extension (const char *path)
-{
-  g_autofree char *basename = NULL;
-  char *p;
-
-  basename = g_path_get_basename (path);
-  p = strrchr (basename, '.');
-  if (g_file_test (path, G_FILE_TEST_IS_DIR) ||
-      !p ||
-      p == basename) /* Leading periods on the basename are ignored. */
-    return NULL;
-  return g_strdup (p + 1);
-}
-
 #ifdef ENABLE_SECCOMP
 static gboolean
 flatpak_fail (GError     **error,
@@ -571,6 +556,18 @@ add_bwrap (GPtrArray   *array,
   if (!g_str_has_prefix (FONTCONFIG_CACHE_PATH, "/usr/"))
     add_args (array, "--ro-bind-try", FONTCONFIG_CACHE_PATH, FONTCONFIG_CACHE_PATH, NULL);
 
+  /*
+   * Used in various distributions. On those distributions, /usr is not
+   * complete without it: some files in /usr might be a symbolic link
+   * like /usr/bin/composite -> /etc/alternatives/composite ->
+   * /usr/bin/composite-im6.q16.
+   *
+   * https://manpages.debian.org/stable/dpkg/update-alternatives.1.en.html
+   * https://docs.fedoraproject.org/en-US/packaging-guidelines/Alternatives/
+   * https://en.opensuse.org/openSUSE:Packaging_Multiple_Version_guidelines
+   */
+  add_args (array, "--ro-bind-try", "/etc/alternatives", "/etc/alternatives", NULL);
+
   add_args (array,
 	    "--proc", "/proc",
 	    "--dev", "/dev",
@@ -784,8 +781,8 @@ script_exec_new (const char  *uri,
   if (exec->sandbox)
     {
       char *tmpl;
-      g_autofree char *ext = NULL;
-      g_autofree char *infile = NULL;
+      const char *infile;
+      g_autofree char *basename = NULL;
 
       exec->fd_array = g_array_new (FALSE, TRUE, sizeof (int));
       g_array_set_clear_func (exec->fd_array, clear_fd);
@@ -799,12 +796,12 @@ script_exec_new (const char  *uri,
           goto bail;
         }
       exec->outfile = g_build_filename (exec->outdir, "gnome-desktop-thumbnailer.png", NULL);
-      ext = get_extension (exec->infile);
+      basename = g_file_get_basename (file);
 
-      if (ext)
-        infile = g_strdup_printf ("gnome-desktop-file-to-thumbnail.%s", ext);
+      if (basename)
+        infile = basename;
       else
-        infile = g_strdup_printf ("gnome-desktop-file-to-thumbnail");
+        infile = "gnome-desktop-file-to-thumbnail";
 
       exec->infile_tmp = g_build_filename (exec->outdir, infile, NULL);
 
