@@ -33,7 +33,7 @@
 #include <sys/stat.h>
 
 #include <glib.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
 #define GNOME_DESKTOP_USE_UNSTABLE_API
@@ -130,8 +130,6 @@ gnome_parse_locale (const char *locale,
         static GRegex *re = NULL;
         GMatchInfo *match_info;
         gboolean    res;
-        gchar      *normalized_codeset = NULL;
-        gchar      *normalized_name = NULL;
         gboolean    retval;
 
         match_info = NULL;
@@ -200,6 +198,9 @@ gnome_parse_locale (const char *locale,
         }
 
         if (codesetp != NULL && *codesetp != NULL) {
+                g_autofree gchar *normalized_codeset = NULL;
+                g_autofree gchar *normalized_name = NULL;
+
                 normalized_codeset = normalize_codeset (*codesetp);
                 normalized_name = construct_language_name (language_codep ? *language_codep : NULL,
                                                            country_codep ? *country_codep : NULL,
@@ -208,11 +209,8 @@ gnome_parse_locale (const char *locale,
 
                 if (language_name_is_valid (normalized_name)) {
                         g_free (*codesetp);
-                        *codesetp = normalized_codeset;
-                } else {
-                        g_free (normalized_codeset);
+                        *codesetp = g_steal_pointer (&normalized_codeset);
                 }
-                g_free (normalized_name);
         }
 
  out:
@@ -288,15 +286,15 @@ gnome_normalize_locale (const char *locale)
 static gboolean
 language_name_is_valid (const char *language_name)
 {
-        gboolean  is_valid;
-        int lc_type_id = LC_MESSAGES;
-        g_autofree char *old_locale = NULL;
+        locale_t locale;
 
-        old_locale = g_strdup (setlocale (lc_type_id, NULL));
-        is_valid = setlocale (lc_type_id, language_name) != NULL;
-        setlocale (lc_type_id, old_locale);
+        locale = newlocale (LC_MESSAGES_MASK, language_name, (locale_t) 0);
+        if (locale != (locale_t) 0) {
+                freelocale (locale);
+                return TRUE;
+        }
 
-        return is_valid;
+        return FALSE;
 }
 
 static void
@@ -304,14 +302,16 @@ language_name_get_codeset_details (const char  *language_name,
                                    char       **pcodeset,
                                    gboolean    *is_utf8)
 {
-        g_autofree char *old_locale = NULL;
-        g_autofree char *codeset = NULL;
+        locale_t locale;
+        locale_t old_locale;
+        const char *codeset = NULL;
 
-        old_locale = g_strdup (setlocale (LC_CTYPE, NULL));
-
-        if (setlocale (LC_CTYPE, language_name) == NULL)
+        locale = newlocale (LC_CTYPE_MASK, language_name, (locale_t) 0);
+        if (locale == (locale_t) 0)
                 return;
 
+        old_locale = uselocale (locale);
+
         codeset = nl_langinfo (CODESET);
 
         if (pcodeset != NULL) {
@@ -319,12 +319,13 @@ language_name_get_codeset_details (const char  *language_name,
         }
 
         if (is_utf8 != NULL) {
-                codeset = normalize_codeset (codeset);
+                g_autofree char *normalized_codeset = normalize_codeset (codeset);
 
-                *is_utf8 = strcmp (codeset, "UTF-8") == 0;
+                *is_utf8 = strcmp (normalized_codeset, "UTF-8") == 0;
         }
 
-        setlocale (LC_CTYPE, old_locale);
+        uselocale (old_locale);
+        freelocale (locale);
 }
 
 /**
@@ -701,12 +702,15 @@ get_translated_language (const char *code,
 
         name = NULL;
         if (language != NULL) {
-                const char  *translated_name;
-                g_autofree char *old_locale = NULL;
+                const char *translated_name;
+                locale_t loc = 0;
+                locale_t old_locale = 0;
 
                 if (locale != NULL) {
-                        old_locale = g_strdup (setlocale (LC_MESSAGES, NULL));
-                        setlocale (LC_MESSAGES, locale);
+                        loc = newlocale (LC_MESSAGES_MASK, locale, (locale_t) 0);
+                        if (loc == (locale_t) 0)
+                                return NULL;
+                        old_locale = uselocale (loc);
                 }
 
                 if (is_fallback_language (code)) {
@@ -719,7 +723,8 @@ get_translated_language (const char *code,
                 }
 
                 if (locale != NULL) {
-                        setlocale (LC_MESSAGES, old_locale);
+                        uselocale (old_locale);
+                        freelocale (loc);
                 }
         }
 
@@ -756,12 +761,15 @@ get_translated_territory (const char *code,
         name = NULL;
         if (territory != NULL) {
                 const char *translated_territory;
-                g_autofree char *old_locale = NULL;
+                locale_t loc;
+                locale_t old_locale;
                 g_autofree char *tmp = NULL;
 
                 if (locale != NULL) {
-                        old_locale = g_strdup (setlocale (LC_MESSAGES, NULL));
-                        setlocale (LC_MESSAGES, locale);
+                        loc = newlocale (LC_MESSAGES_MASK, locale, (locale_t) 0);
+                        if (loc == (locale_t) 0)
+                                return NULL;
+                        old_locale = uselocale (loc);
                 }
 
                 translated_territory = dgettext ("iso_3166", territory);
@@ -769,7 +777,8 @@ get_translated_territory (const char *code,
                 name = capitalize_utf8_string (tmp);
 
                 if (locale != NULL) {
-                        setlocale (LC_MESSAGES, old_locale);
+                        uselocale (old_locale);
+                        freelocale (loc);
                 }
         }
 
@@ -788,6 +797,7 @@ languages_parse_start_tag (GMarkupParseContext      *ctx,
         const char *ccode_longT;
         const char *ccode;
         const char *ccode_id;
+        const char *lang_common_name;
         const char *lang_name;
 
         if (! (g_str_equal (element_name, "iso_639_entry") || g_str_equal (element_name, "iso_639_3_entry"))
@@ -799,6 +809,7 @@ languages_parse_start_tag (GMarkupParseContext      *ctx,
         ccode_longB = NULL;
         ccode_longT = NULL;
         ccode_id = NULL;
+        lang_common_name = NULL;
         lang_name = NULL;
 
         while (*attr_names && *attr_values) {
@@ -835,6 +846,11 @@ languages_parse_start_tag (GMarkupParseContext      *ctx,
                                 }
                                 ccode_id = *attr_values;
                         }
+                } else if (g_str_equal (*attr_names, "common_name")) {
+                        /* skip if empty */
+                        if (**attr_values) {
+                                lang_common_name = *attr_values;
+                        }
                 } else if (g_str_equal (*attr_names, "name")) {
                         lang_name = *attr_values;
                 }
@@ -843,6 +859,10 @@ languages_parse_start_tag (GMarkupParseContext      *ctx,
                 ++attr_values;
         }
 
+        if (lang_common_name != NULL) {
+                lang_name = lang_common_name;
+        }
+
         if (lang_name == NULL) {
                 return;
         }
@@ -1001,6 +1021,9 @@ languages_init (void)
         if (gnome_languages_map)
                 return;
 
+        bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+        bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+
         gnome_languages_map = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
 
         languages_variant_init ("iso_639");
@@ -1073,6 +1096,8 @@ gnome_get_language_from_locale (const char *locale,
         g_autofree char *langinfo_codeset = NULL;
         g_autofree char *translated_language = NULL;
         g_autofree char *translated_territory = NULL;
+        g_autofree char *modifier = NULL;
+        g_autofree char *translated_modifier = NULL;
         gboolean is_utf8 = TRUE;
 
         g_return_val_if_fail (locale != NULL, NULL);
@@ -1087,7 +1112,7 @@ gnome_get_language_from_locale (const char *locale,
                             &language_code,
                             &territory_code,
                             &codeset_code,
-                            NULL);
+                            &modifier);
 
         if (language_code == NULL) {
                 goto out;
@@ -1104,6 +1129,11 @@ gnome_get_language_from_locale (const char *locale,
 		goto out;
 	}
 
+        if (modifier != NULL)
+                translated_modifier = gnome_get_translated_modifier (modifier, translation);
+        if (translated_modifier != NULL)
+                g_string_append_printf (full_language, " — %s", translated_modifier);
+
         if (territory_code != NULL) {
                 translated_territory = get_translated_territory (territory_code, translation);
         }
@@ -1158,6 +1188,8 @@ gnome_get_country_from_locale (const char *locale,
         g_autofree char *langinfo_codeset = NULL;
         g_autofree char *translated_language = NULL;
         g_autofree char *translated_territory = NULL;
+        g_autofree char *modifier = NULL;
+        g_autofree char *translated_modifier = NULL;
         gboolean is_utf8 = TRUE;
 
         g_return_val_if_fail (locale != NULL, NULL);
@@ -1172,7 +1204,7 @@ gnome_get_country_from_locale (const char *locale,
                             &language_code,
                             &territory_code,
                             &codeset_code,
-                            NULL);
+                            &modifier);
 
         if (territory_code == NULL) {
                 goto out;
@@ -1190,10 +1222,18 @@ gnome_get_country_from_locale (const char *locale,
         }
         if (translated_language != NULL) {
                 g_string_append_printf (full_name,
-                                        " (%s)",
+                                        " (%s",
                                         translated_language);
         }
 
+        if (modifier != NULL)
+                translated_modifier = gnome_get_translated_modifier (modifier, translation);
+        if (translated_modifier != NULL)
+                g_string_append_printf (full_name, " — %s", translated_modifier);
+
+        if (translated_language != NULL)
+                g_string_append_printf (full_name, ")");
+
         language_name_get_codeset_details (translation, &langinfo_codeset, &is_utf8);
 
         if (codeset_code == NULL && langinfo_codeset != NULL) {
@@ -1256,7 +1296,7 @@ gnome_get_all_locales (void)
  * @code: an ISO 639 code string
  * @translation: (allow-none): a locale string
  *
- * Gets the language name for @code. If @locale is provided the
+ * Gets the language name for @code. If @translation is provided the
  * returned string is translated accordingly.
  *
  * Return value: (transfer full): the language name. Caller takes
@@ -1280,7 +1320,7 @@ gnome_get_language_from_code (const char *code,
  * @code: an ISO 3166 code string
  * @translation: (allow-none): a locale string
  *
- * Gets the country name for @code. If @locale is provided the
+ * Gets the country name for @code. If @translation is provided the
  * returned string is translated accordingly.
  *
  * Return value: (transfer full): the country name. Caller takes
@@ -1299,6 +1339,82 @@ gnome_get_country_from_code (const char *code,
         return get_translated_territory (code, translation);
 }
 
+/**
+ * gnome_get_translated_modifier:
+ * @modifier: the modifier part of a locale name
+ * @translation: (allow-none): a locale string
+ *
+ * Gets a translation of the raw @modifier string. If @translation
+ * is provided the returned string is translated accordingly.
+ *
+ * Return value: (transfer full): the translated modifier string.
+ * Caller takes ownership.
+ *
+ * Since: 3.34
+ */
+char *
+gnome_get_translated_modifier (const char *modifier,
+                               const char *translation)
+{
+        char *retval;
+        GHashTable *modifiers_map;
+        locale_t loc;
+        locale_t old_locale;
+
+        g_return_val_if_fail (modifier != NULL, NULL);
+
+        if (translation != NULL) {
+                loc = newlocale (LC_MESSAGES_MASK, translation, (locale_t) 0);
+                if (loc == (locale_t) 0) {
+                        return NULL;
+                }
+                old_locale = uselocale (loc);
+        }
+
+        /* Modifiers as listed in glibc's SUPPORTED file:
+         * https://sourceware.org/git/?p=glibc.git;a=blob;f=localedata/SUPPORTED;hb=HEAD
+         * (except for @euro, which would be superfluous in this context). */
+        modifiers_map = g_hash_table_new (g_str_hash, g_str_equal);
+
+        /* TRANSLATORS: Used to distinguish the labels representing the gez_ER
+           and gez_ET locales from gez_ER@abegede respective gez_ET@abegede. The
+           difference is related to collation. */
+        g_hash_table_insert (modifiers_map, "abegede", _("Abegede"));
+        /* TRANSLATORS: Used to distinguish Cyrillic from Latin written language variants. */
+        g_hash_table_insert (modifiers_map, "cyrillic", _("Cyrillic"));
+        /* TRANSLATORS: Also known as "Nagari", a written variant for many languages
+           of the Indian subcontinent. See:
+           https://en.wikipedia.org/wiki/Devanagari */
+        g_hash_table_insert (modifiers_map, "devanagari", _("Devanagari"));
+        /* TRANSLATORS: Used to distinguish the label representing the tt_RU
+           locale from tt_RU@iqtelif. It's a special alphabet for Tatar. */
+        g_hash_table_insert (modifiers_map, "iqtelif", _("IQTElif"));
+        /* TRANSLATORS: The alphabet/script, not the language. Used to distinguish
+           Latin from Cyrillic written language variants. */
+        g_hash_table_insert (modifiers_map, "latin", _("Latin"));
+        /* TRANSLATORS: "Saho" is a variant of the Afar language. Used to
+           distinguish the label representing the aa_ER locale from aa_ER@saaho. */
+        g_hash_table_insert (modifiers_map, "saaho", _("Saho"));
+        /* TRANSLATORS: "Valencia" is a dialect of the Catalan language spoken
+           in Valencia. Used to distinguish the label representing the ca_ES
+           locale from ca_ES@valencia. */
+        g_hash_table_insert (modifiers_map, "valencia", _("Valencia"));
+
+        if (g_hash_table_contains (modifiers_map, modifier))
+                retval = g_strdup (g_hash_table_lookup (modifiers_map, modifier));
+        else
+                retval = g_strdup (modifier);
+
+        g_hash_table_destroy (modifiers_map);
+
+        if (translation != NULL) {
+                uselocale (old_locale);
+                freelocale (loc);
+        }
+
+        return retval;
+}
+
 /**
  * gnome_get_input_source_from_locale:
  * @locale: a locale string
